﻿using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.Script.Serialization;



namespace SamplePluginUsingOAuth
{
    /// <summary>
    /// Quick mockup plugin to test functionality about different sso version and calling the restapi available to plugins using oauth + new sso 
    /// </summary>
    public partial class Default : Page
    {
        private const string PluginAuthPlaceHolder = "{itsl_auth}";
        private const string ItslAuthParameterName = "itsl_auth";
        private const string ItslSignParameterName = "itsl_sign";
        private const string ItslAuthParamTimeStamp = "TimeStamp";
        private const string ItslAuthParamOauthToken = "OAuthToken";
        private const string ItslAuthParamOauthTokenSecret = "OAuthTokenSecret";
        private const string ItslAuthParamPostTo = "PostTo";
        private string _oauthToken = string.Empty;
        private string _oauthTokenSecret = string.Empty;
        private int _personId;

        private const string RunningInIFrameParam = "RunningInIFrame";
        protected void Page_Init(object sender, EventArgs e)
        {
            NameValueCollection queryStringCollection = Context.Request.QueryString;
            // only valid for new sso version

            JavaScriptSerializer serializer = new JavaScriptSerializer();
            string jsonString = HttpUtility.UrlDecode(queryStringCollection[ItslAuthParameterName]);
            ItslAuthDeserialized = serializer.Deserialize<Dictionary<string, object>>(jsonString);
            foreach (var o in ItslAuthDeserialized)
            {
                ItslAuthParamsList.Items.Add(string.Format("{0} : {1}", o.Key, o.Value));
                if(o.Key==ItslAuthParamOauthToken)
                {
                    _oauthToken = o.Value != null ? o.Value.ToString() : "null";
                }
                if (o.Key == ItslAuthParamOauthTokenSecret)
                {
                    _oauthTokenSecret = o.Value != null ? o.Value.ToString() : "null";
                }
            }
            _personId = int.Parse(ItslAuthDeserialized["PersonId"].ToString());
            // Check with new signature method
            bool isValidSso = SignatureValidator.IsPluginSignatureValidNewSso(jsonString,
                                                                              AppSettings.ItslPluginSharedSecret,
                                                                              AppSettings.SignatureTimeoutInSeconds,
                                                                              queryStringCollection[ItslSignParameterName],
                                                                              ItslAuthDeserialized[ItslAuthParamTimeStamp].ToString());
            
            // Get querystring and show it
            RawUrl.Text = string.Format("Raw url to plugin is urldecoded: <b>{0}</b>", HttpUtility.UrlDecode(Context.Request.RawUrl));
            SignatureIsValidString.Text = isValidSso ? "Signature is valid!" : "Signature is invalid!";
            SsoVersionLabel.Text = string.Format("Uses {0} version of sso", UsingNewSsoVersion ? "new" : "old");
            
            foreach (string key in queryStringCollection.AllKeys)
            {
                ParamsList.Items.Add(string.Format("{0} - {1}", key, queryStringCollection[key]) );
            }

            // Setup url for iframe to be sent as html content
            var iFrameUri = new UriBuilder(Request.Url.Scheme, Request.Url.Host, Request.Url.Port, ResolveUrl("Default.aspx"));
            IFrameSource.Value = iFrameUri.ToString() + '?' + RunningInIFrameParam + "=true&" + PluginAuthPlaceHolder;
            PostToValue.Value = PostToParameter;
            // Don't show insert button in "render mode", i.e. that it's learning is rendering the iframe
            InsertButton.Visible = IsAddMode;

            CallRestApiAndOutputResponse();
        }

        private void CallRestApiAndOutputResponse()
        {
            AppServiceRestClient restClient = new AppServiceRestClient();
            LicensesForUserResponse = HttpUtility.HtmlEncode(restClient.GetLicensesForUser(_oauthToken, _oauthTokenSecret));
            OrganisationsForUserResponse = HttpUtility.HtmlEncode(restClient.GetOrganisationsForUser(_oauthToken, _oauthTokenSecret, _personId));
            CurrentUserResponse = HttpUtility.HtmlEncode(restClient.GetPerson(_oauthToken, _oauthTokenSecret));
        }

        public IDictionary<string, object> ItslAuthDeserialized { get; set; }
        public bool UsingNewSsoVersion
        {
            get
            {
                return Context.Request.QueryString.AllKeys.Contains(ItslAuthParameterName);
            }
        }

        public string PostToParameter
        {
            get
            {
                if(UsingNewSsoVersion)
                {
                    return HttpUtility.UrlDecode(ItslAuthDeserialized[ItslAuthParamPostTo].ToString());
                }
                return HttpUtility.UrlDecode(Context.Request.QueryString[ItslAuthParamPostTo]);
            }
        }

        /// <summary>
        /// Is true if plugin is launched through add plugin in fckeditor or add content block
        /// </summary>
        public bool IsAddMode
        {
            get
            {
                return Context.Request.QueryString[RunningInIFrameParam] == null;
            }
        }

        protected string LicensesForUserResponse
        {
            get; set;
        }

        protected string OrganisationsForUserResponse
        {
            get; set;
        }

        protected string CurrentUserResponse
        {
            get; set;
        }
    }
}
